@@ -32,6 +32,7 @@
 #include <string>         // for string
 #include <thread>         // for get_id, thread, thread::id
 #include <unordered_map>  // for unordered_map
+#include <unordered_set>  // for unordered_set
 #include <utility>        // for pair
 #include <vector>         // for vector
 
@@ -51,6 +52,13 @@
 #define REPORT(...) \
   (REPORTER.report() ? REPORTER(__VA_ARGS__).prefix(this) : REPORTER)
 
+#ifdef LIBSEMIGROUPS_FMT_ENABLED
+#define REPORT_DEFAULT_V3(...) \
+  REPORTER(__VA_ARGS__).prefix(nullptr).thread_color().flush()
+#else
+#define REPORT_DEFAULT_V3(...) REPORTER(__VA_ARGS__).prefix(nullptr).flush();
+#endif
+
 #ifdef LIBSEMIGROUPS_FMT_ENABLED
 #define REPORT_DEFAULT(...) REPORT(__VA_ARGS__).thread_color().flush();
 #else
@@ -96,10 +104,10 @@ namespace libsemigroups {
     class ThreadIdManager final {
      public:
       ThreadIdManager();
-      ThreadIdManager(ThreadIdManager const&) = delete;
-      ThreadIdManager(ThreadIdManager&&)      = delete;
+      ThreadIdManager(ThreadIdManager const&)            = delete;
+      ThreadIdManager(ThreadIdManager&&)                 = delete;
       ThreadIdManager& operator=(ThreadIdManager const&) = delete;
-      ThreadIdManager& operator=(ThreadIdManager&&) = delete;
+      ThreadIdManager& operator=(ThreadIdManager&&)      = delete;
 
       void   reset();
       size_t tid(std::thread::id t);
@@ -298,24 +306,31 @@ namespace libsemigroups {
      public:
       explicit Reporter(bool report = true);
 
-      Reporter(Reporter const&) = delete;
-      Reporter(Reporter&&)      = delete;
+      Reporter(Reporter const&)            = delete;
+      Reporter(Reporter&&)                 = delete;
       Reporter& operator=(Reporter const&) = delete;
-      Reporter& operator=(Reporter&&) = delete;
+      Reporter& operator=(Reporter&&)      = delete;
 
       template <typename TClass>
-      Reporter& prefix(TClass const* const ptr, bool overide = false) {
-        if (_report || overide) {
+      Reporter& prefix(TClass const* const ptr, bool override = false) {
+        if (_report || override) {
           std::lock_guard<std::mutex> lg(_mtx);
           uint64_t tid = THREAD_ID_MANAGER.tid(std::this_thread::get_id());
           resize(tid + 1);
           _options[tid].prefix
 #ifdef LIBSEMIGROUPS_FMT_ENABLED
-              = fmt::sprintf("#%llu: %s: ", tid, string_class_name(ptr));
+              = fmt::sprintf("#%llu: ", tid);
 #else
-              = string_format(
-                  "#%llu: %s: ", tid, string_class_name(ptr).c_str());
+              = string_format("#%llu: ", tid);
 #endif
+          if (ptr != nullptr) {
+            _options[tid].prefix +=
+#ifdef LIBSEMIGROUPS_FMT_ENABLED
+                fmt::sprintf("%s: ", string_class_name(ptr));
+#else
+                string_format("%s: ", string_class_name(ptr).c_str());
+#endif
+          }
         }
         return *this;
       }
@@ -324,9 +339,23 @@ namespace libsemigroups {
       Reporter& color(fmt::color);
       Reporter& thread_color();
 #endif
-
       Reporter& prefix();
 
+      Reporter& prefix(std::nullptr_t) {
+        if (_report) {
+          std::lock_guard<std::mutex> lg(_mtx);
+          uint64_t tid = THREAD_ID_MANAGER.tid(std::this_thread::get_id());
+          resize(tid + 1);
+          _options[tid].prefix
+#ifdef LIBSEMIGROUPS_FMT_ENABLED
+              = fmt::sprintf("#%llu: ", tid);
+#else
+              = string_format("#%llu: ", tid);
+#endif
+        }
+        return *this;
+      }
+
       std::string get_prefix() const noexcept {
         uint64_t tid = THREAD_ID_MANAGER.tid(std::this_thread::get_id());
         return _options[tid].prefix;
@@ -364,6 +393,16 @@ namespace libsemigroups {
         return *this;
       }
 
+      Reporter& suppress(std::string const& class_name) {
+        _suppressions.insert(class_name);
+        return *this;
+      }
+
+      Reporter& clear_suppressions() {
+        _suppressions.clear();
+        return *this;
+      }
+
      private:
       void resize(size_t);
 
@@ -382,11 +421,12 @@ namespace libsemigroups {
         bool        flush_right;
         std::string prefix;
       };
-      std::vector<std::string> _last_msg;
-      std::mutex               _mtx;
-      std::vector<std::string> _msg;
-      std::vector<Options>     _options;
-      std::atomic<bool>        _report;
+      std::vector<std::string>        _last_msg;
+      std::mutex                      _mtx;
+      std::vector<std::string>        _msg;
+      std::vector<Options>            _options;
+      std::atomic<bool>               _report;
+      std::unordered_set<std::string> _suppressions;
     };
   }  // namespace detail
 
@@ -464,7 +504,9 @@ namespace libsemigroups {
 
   namespace report {
     bool should_report() noexcept;
-  }
+    void suppress(std::string const&);
+    void clear_suppressions();
+  }  // namespace report
 
   //! This struct can be used to enable printing of some information during
   //! various of the computation in ``libsemigroups``. Reporting is enable (or
